import axios from "axios";
import set from "lodash/set";
import get from "lodash/get";
import config from "../auth_config.json";
import keys from "../api_keys.json";

/**
 * Update or add data at given path in custom data.
 * @param {string} path what should be updated, requires at least team.dataset.
 * @param {object} payload data payload that'll be saved on path.
 * @param {add|replace} method either add or replace.
 * @example:
 * API.updateData("professors.example", {some: "value"}) // merging with previous data
 * API.updateData("professors.example", {different: "value"}, "replace") // rewriting data
 */
const updateData = (api) => (path, payload, method = "add") => {
  if (!path) {
    throw new Error("Error, path must be specified when calling updateData endpoint");
  }
  if (path.split(".").length < 2) {
    throw new Error(`Error, path must have proper namespace in format "team.dataset".`);
  }
  if (!method || !["replace", "add"].includes(method)) {
    throw new Error(`Update method is not specified. It muset be one of ["replace", "add"]`);
  }
  return apiInstance.get("/rest/customdata").then((data) => {
    const [namespace, ...paths] = path.split(".");
    const destination = paths.join("");
    const index = data.findIndex(({ team }) => team === namespace);
    if (index === -1) {
      throw new Error(`${target} is not valid namepsace. Available namespaces ${data.map(({ team }) => team)}`);
    }
    const target = `[${index}].data.${destination}`;

    return api.post(
      "/rest/customdata",
      method === "replace" ? set(data, target, payload) : set(data, `${target}[${[...(get(data, target) || [])].length}]`, payload)
    );
  });
};

/**
 * Retrieve custom data for specific team.
 * @param {string} teamName team to read data from.
 */
const getCustomData = (api) => (teamName = "professors") => {
  return api.get(`/rest/customdata?q={"team": "${teamName}"}`);
};

const instances = Object.entries(keys).reduce(
  (acc, [key, api]) => ({
    ...acc,
    [key]: {
      apiInstance: axios.create({
        baseURL: api.apiBase,
        headers: {
          "content-type": "application/json",
          "x-apikey": api.key,
          "cache-control": "no-cache"
        }
      })
    }
  }),
  {}
);

Object.keys(instances).forEach((instanceKey) => {
  instances[instanceKey].apiInstance.interceptors.response.use((response) => response.data || response);
  instances[instanceKey].get = (path) => instances[instanceKey].apiInstance.get(path);
  instances[instanceKey].post = (path, payload) => instances[instanceKey].apiInstance.post(path, payload);
  instances[instanceKey].updateData = updateData(instances[instanceKey].apiInstance);
  instances[instanceKey].getCustomData = getCustomData(instances[instanceKey].apiInstance);
});

const apiHeaders = {
  "content-type": "application/json",
  "x-apikey": config.apiKey,
  "cache-control": "no-cache"
};

const apiInstance = axios.create({
  baseURL: config.apiBase,
  headers: apiHeaders
});

apiInstance.interceptors.response.use((response) => response.data || response);

const Api = {
  ...instances,
  updateData: updateData(apiInstance),
  getCustomData: getCustomData(apiInstance),
  get: (path) => apiInstance.get(path),
  post: (path, payload) => apiInstance.post(path, payload)
};

export default Api;
